﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PMDotNet;
using System.Runtime.InteropServices;

namespace PokerAlwaysCall
{
    class PMSimpleBot
    {
      
        static PMController pmController;

        private static bool isRunning = true;

        static string serverIP="127.0.0.1";
        static string serverPort="8088";
        static string playerName="testuser";
        static string playerPassword="password";
        static int botType=2;

        static Random random = new Random();

        /* This portion of code, not related to the Poker Server.
         * Just the way to handle console application close event
         */ 
         
        [DllImport("Kernel32")]
        public static extern bool SetConsoleCtrlHandler(HandlerRoutine Handler, bool Add);

        public delegate bool HandlerRoutine(CtrlTypes CtrlType);

        public enum CtrlTypes
        {
            CTRL_C_EVENT = 0,
            CTRL_BREAK_EVENT,
            CTRL_CLOSE_EVENT,
            CTRL_LOGOFF_EVENT = 5,
            CTRL_SHUTDOWN_EVENT
        }
        
        static void Main(string[] args)
        {



            Console.Write("Please Enter Server IP: ");
            serverIP = Console.ReadLine();

            Console.Write("Please Enter Server Port: ");
            serverPort = Console.ReadLine();

            Console.Write("Please Enter User Name: ");
            playerName = Console.ReadLine();

            Console.Write("Please Enter User Password: ");
            playerPassword = Console.ReadLine();


            Console.Write("Please Select Bot Type (1-Random,2-Always Call,3-Random Aggressive): ");

            try
            {
                botType = Convert.ToInt32(Console.ReadLine());
            }
            catch (Exception ex)
            {

                botType = 2;
            }

            if (botType < 1 || botType > 3) botType = 2;


            SetConsoleCtrlHandler(new HandlerRoutine(ConsoleCtrlCheck), true);

            pmController = new PMController(serverIP, serverPort, "2.91");

            pmController.OnPMServerConnected += new PMController.PMObjHandler(OnPMServerConnected);
            pmController.OnPMServerDisconnected += new PMController.PMObjHandler(OnPMServerDisconnected);
            pmController.OnPMServerLogin += new PMController.PMObjHandler(OnPMServerLogin);
            pmController.OnPMServerRingTablesData += new PMController.PMObjHandler(OnPMServerRingTablesData);
            pmController.OnPMServerPlayersData += new PMController.PMObjHandler(OnPMServerPlayersData);
            pmController.OnPMTableMessage += new PMController.PMObjStringHandler(OnPMTableMessage);
            pmController.OnPMTableAction += new PMController.PMObjHandler(OnPMTableAction);
            pmController.OnPMTableSeatChoice += new PMController.PMObjHandler(OnPMTableSeatChoice);
            pmController.OnPMTableSeatReserved += new PMController.PMObjIntegerHandler(OnPMTableSeatReserved);
            pmController.OnPMTableSeatConfirmed += new PMController.PMObjIntegerHandler(OnPMTableSeatConfirmed);
            pmController.OnPMTableChatMessage += new PMController.PMObjStringsHandler(OnPMTableChatMessage);
        
            pmController.PMConnect();

            

            while (isRunning) ;

        }

        private static void OnPMServerDisconnected(Object sender)
        {

          
            Console.WriteLine("Disconnected from the server");
            Console.WriteLine("Press any key to close application");
            Console.ReadKey();
            isRunning = false; //To close console application
        }

        private static void OnPMServerConnected(Object sender)
        {
            //Successfully connected to the server

            // PMSession - for now empty class, just general information about poker server.
            // Not necessary information for the bots.
            
            PMSession currentSession = sender as PMSession;

            //Send login and password to the server for authorization

            pmController.PMLogin(playerName, playerPassword);

        }

        private static void OnPMServerLogin(Object sender)
        {
            //Login accepted. PMPlayer class information about logged in user.
            PMPlayer currentPlayer = sender as PMPlayer;

            // Retrive data about tables.
            List<PMTable> tables = pmController.PMGetRingTables();

            // Sending request to join table. In demo version can join only first table.
            pmController.PMJoinRingTable(1);
         
        }

        private static void OnPMServerRingTablesData(Object sender)
        {
            //Server sends to us information about all ring tables

            List<PMTable> ringTables = sender as List<PMTable>;

         
        }

        private static void OnPMServerPlayersData(Object sender)
        {
            // Server sends information about all players

            List<PMPlayer> players = sender as List<PMPlayer>;

        }

        private static void OnPMTableMessage(Object sender, string message)
        {
         
            //GUI Messages from the server. Nothing important.

                PMTable table = sender as PMTable;

                Console.WriteLine(message);
                     

        }

        static private void OnPMTableSeatChoice(Object sender)
        {

            PMTable tables = sender as PMTable;

            //Calling method to retrieve available seats for current table
            List<int> freeseats = pmController.PMGetAvailableSeats(1);

            if (freeseats.Count > 0)
            {
                //Get first available seat
                int freeseat = freeseats[0];
                //Calling server to request selected seat
                pmController.PMRequestSeat(1, freeseat);
            }



        }

        static private void OnPMTableSeatReserved(Object sender, int seat)
        {

            // Server inform us that selected seat reserved.
            // In OnPMTableSeatConfirmed event server send all neccessary for us information

            PMTable table = sender as PMTable;

        }

        static private void OnPMTableSeatConfirmed(Object sender, int balance)
        {
            //Server inform us that selected seat reserved.
            //In instance of PMTable class we can retrive Min BuyIn, Max BuyIn and Default BuyIn data for selected table
            PMTable table = sender as PMTable;
            //Sends to the server BuyIn agreement for previously selected seat and table.
            pmController.PMSeatPlayer(table.ID, table.DefBuyIn);


        }

        static private void OnPMTableChatMessage(Object sender, string who, string message)
        {

                PMTable table = sender as PMTable;
                
                if (!String.IsNullOrEmpty(message))
                {
                    Console.WriteLine(who + ": " + message);                    
                }

                else
                {
                    Console.WriteLine();                    
                }
        } 

        static private  void OnPMTableAction(Object sender)
        {

            PMAction currentAction = sender as PMAction;
          
            if (String.IsNullOrEmpty(currentAction.Button1) || String.IsNullOrEmpty(currentAction.Button3)){
            
                 pmController.PMSendCommand(currentAction.Button2);
            
                 return;
            } 
            


            int botRandom;

            
            switch (botType) 
            { 
            
                case 1: //Random bot, sends to the server random selection.
                    
                //Random number 1 to 3, and from that value send to the server selected command
                   
                    botRandom= random.Next(1, 4);

                    switch (botRandom) { 
                    
                        case 1:
                            
                            pmController.PMSendCommand(currentAction.Button1);
                            break;

                        case 2:
                            pmController.PMSendCommand(currentAction.Button2);
                            break;

                        case 3:
                            pmController.PMSendCommand(currentAction.Button3, currentAction.MinRaise);
                            break;                    
                    }

                    break;

                case 2: //Always call bot. Just always send to the server check or call

                    pmController.PMSendCommand(currentAction.Button2);
                    break;

                case 3: //Aggressive bot, never fold.

                    //Random number 2 to 3, and from that value send to the server selected command
                   
                    botRandom= random.Next(2, 4);

                     switch (botRandom) { 
                                         
                        case 2:
                            pmController.PMSendCommand(currentAction.Button2);
                            break;

                        case 3:
                            pmController.PMSendCommand(currentAction.Button3, currentAction.MinRaise);
                             
                            break;                    
                    }

                    break;

            
            }

          

        }

    

        private static bool ConsoleCtrlCheck(CtrlTypes ctrlType)
        {
            pmController.PMLeaveTable();
            pmController.PMDisconnect();
               
            return true;
        }
    }
}
